if (typeof(Drupal) == "undefined" || !Drupal.tribune) {
   Drupal.tribune = {tribunes: {}};
}

Drupal.behaviors.tribune = function() {
	for (var element_id in Drupal.settings.tribune.tribunes) (function (element_id) {
		var tribune = {
			id: element_id,
			highlightedElements: {},
			postElements: {},
			element: $(".tribune#tribune-" + element_id),
			form: $(".tribune#tribune-" + element_id + " form.tribune-post-form"),
			palmipede: $(".tribune#tribune-" + element_id + " .tribune-palmipede"),
			pinnipede: $(".tribune#tribune-" + element_id + " .tribune-pinnipede"),
			preview: $(".tribune#tribune-" + element_id + " .tribune-preview"),
			error: $(".tribune#tribune-" + element_id + " .tribune-error-message"),
			settings: Drupal.settings.tribune.tribunes[element_id],
			last_load_time: Drupal.settings.tribune.tribunes[element_id].time,
			ajax_preview_request: undefined
		};

		tribune.pinnipede.find('li').each(function() {
			var post_id = $(this).attr("value");
			tribune.postElements[post_id] = $(this);
		});

		tribune.is_element_visible = function(jqelement) {
			element = jqelement.get(0);

			var scrollY = 0;
			if (window.pageYOffset) {
				scrollY = window.pageYOffset;
			} else if (document.body && document.body.scrollTop) {
				scrollY = document.body.scrollTop;
			} else if (document.documentElement && document.documentElement.scrollTop) {
				scrollY = document.documentElement.scrollTop;
			}

			var posY = 0;
			if (element.offsetParent) {
				for (var posY = 0 ; element.offsetParent ; element = element.offsetParent) {
					posY += element.offsetTop;
				}
			} else {
				posY = element.y;
			}

			return posY > scrollY;
		}

		tribune.set_error = function(request, text, error) {
			tribune.element.addClass("tribune-error");
			var message = null;
			switch (text) {
				case "error":
					message = text;
					if (request.status) {
						message += ": " + request.status;
						if (request.statusText) {
							message += " - " + request.statusText;
						}
					} else {
						message += ": could not connect to server";
					}
					break;
				default:
					if (console && console.log) {
						console.log(request);
						console.log(text);
						console.log(error);
					}
					message = text;
					break
			}

			if (message) {
				tribune.error.html(message);
				tribune.error.fadeIn();
			}
		}

		tribune.clear_error = function() {
			tribune.element.removeClass("tribune-error");
			tribune.error.fadeOut();
		}

		tribune.slip = function(message) {
			regex = /^(.*?)<(m|s|u|b|i|tt)>(.*?)<\/\2>(.*?)$/;
			var matches = regex.exec(message);
			
			while (matches) {
				message = matches[1] + '\026' + matches[2] + '\027' + tribune.slip(matches[3]) + '\026/' + matches[2] + '\027' + matches[4];

				matches = regex.exec(message);
			};

			return message;
		}

		tribune.show_preview = function() {
			var message = tribune.palmipede.val();
			if (message != '') {
				message = tribune.slip(message);
				message = message.replace(/</g, '&lt;');
				message = message.replace(/>/g, '&gt;');
				message = message.replace(/\026/g, '<');
				message = message.replace(/\027/g, '>');
				message = message.replace(/((([01]?[0-9])|(2[0-3])):([0-5][0-9])(:([0-5][0-9]))?([:\^][0-9]|¹|²|³)?(@[0-9A-Za-z]+)?)/g, '<a href="#" class="tribune-clock">$1</a>');
				//message = message.replace(/(http:\/\/.*)(\s)/, '<a href="$1">[url]</a>$2');
				tribune.show_popup($("<span class='tribune-message'>" + message + "</span>"));
				tribune.hide_popup(2000);
			} else {
				tribune.hide_popup(0);
			}
		}

		tribune.show_popup = function(post) {
			if (post) {
				tribune.preview.html(post.html());
				tribune.preview.fadeIn(50);
				if (tribune.preview_timeout) {
					clearTimeout(tribune.preview_timeout);
					tribune.preview_timeout = undefined;
				}
			}
		}

		tribune.hide_popup = function(timeout) {
			if (timeout == undefined) {
				timeout = 500;
			}

			if (tribune.ajax_preview_request && tribune.ajax_preview_request.abort) {
				tribune.ajax_preview_request.abort();
			}

			if (tribune.preview_timeout) {
				clearTimeout(tribune.preview_timeout);
			}
			tribune.preview_timeout = setTimeout(function() {tribune.preview.fadeOut(200); tribune.preview_timeout = undefined;}, timeout);
		}

		tribune.reload_success = function(json) {
			if (json) {
				tribune.clear_error();

				var new_posts = [];

				tribune.last_load_time = json.time;

				for (post_id in json.posts) {
					existing_post = tribune.postElements[post_id];

					if (json.posts[post_id].text) {
						post = $(json.posts[post_id].text);

						if (existing_post) {
							existing_post.replaceWith(post);
							tribune.postElements[post_id] = post;
						} else if (post.attr("value") > tribune.pinnipede.find('li:last').attr("value") || tribune.pinnipede.find('li').length == 0) {
							var first_id = tribune.pinnipede.find('li:first').attr("value");

							if (first_id && post_id >= first_id) while (tribune.pinnipede.find("li").length >= tribune.settings.history_size) {
								var first = tribune.pinnipede.find('li:first');

								delete tribune.postElements[first.attr("value")];
								first.remove();
							}

							post.appendTo(tribune.pinnipede);
							tribune.postElements[post_id] = post;
						}

						new_posts.push('#' + post.attr('id'));

					} else if (existing_post) {
						existing_post.remove();
						delete tribune.postElements[post_id];
					}
				}

				for (i in tribune.settings.after_reload) {
					tribune.settings.after_reload[i](tribune.pinnipede.find(new_posts.join(',')));
				}
			}
		}

		tribune.reload_complete = function(json) {
			tribune.element.removeClass("tribune-reloading");
			tribune.pinnipede.find("input").removeAttr("disabled").removeAttr("readonly");
			tribune.start_timer();
		}

		tribune.load_new_posts = function() {
			tribune.element.addClass("tribune-reloading");
			tribune.pinnipede.find("input").attr("disabled", "disabled").attr("readonly", "readonly");
			tribune.stop_timer();

			$.ajax({
				url: tribune.settings.json_posts_url + "/" + tribune.last_load_time,
				dataType: "json",
				cache: false,
				success: tribune.reload_success,
				error: tribune.set_error,
				complete: tribune.reload_complete
			});
		}

		tribune.palmipede.set_selection_range = function(start, end) {
			field = tribune.palmipede.get(0);

			if (field.setSelectionRange) {
				field.setSelectionRange(start, end);
			} else if (field.createTextRange) {
				var range = field.createTextRange();
				range.collapse(true);
				range.moveStart('character', start);
				range.moveEnd('character', end - start);
				range.select();
			}
		}

		tribune.palmipede.get_selection_range = function() {
			field = tribune.palmipede.get(0);

			if (field.setSelectionRange) {
				return [field.selectionStart, field.selectionEnd];
			} else if (field.createTextRange) {
				var range = document.selection.createRange();

				if (range.parentElement() == field) {
					var range2 = field.createTextRange();
					range2.collapse(true);
					range2.setEndPoint('EndToEnd', range);
					return [range2.text.length - range.text.length, range2.text.length];
				}
			}
			return [field.value.length, field.value.length];
		}

		tribune.insert_into_palmipede = function(text) {
			var message = tribune.palmipede.val();
			var range = tribune.palmipede.get_selection_range();
			tribune.palmipede.get(0).focus();
			tribune.palmipede.val(message.substring(0, range[0]) + text + message.substring(range[1], message.length));
			tribune.palmipede.set_selection_range(range[0] + text.length, range[0] + text.length);
			tribune.palmipede.change();
		}

		tribune.highlight_element = function(element) {
			element.addClass('tribune-highlighted');
			tribune.highlightedElements[element.attr('id')] = element;
		}

		tribune.unhighlight_all_posts = function() {
			for (post_id in tribune.highlightedElements) {
				tribune.highlightedElements[post_id].removeClass('tribune-highlighted');
				delete tribune.highlightedElements[post_id];
			}

			tribune.hide_popup();
		}

		tribune.highlight_main_post = function(post_id) {
			var highlight_post = function(post) {
				post.addClass('tribune-highlighted');
				tribune.highlightedElements[post.attr('id')] = post;

				post.find('link[rel=post-reference]').each(function() {
					tribune.highlight_element(tribune.pinnipede.find('a' + $(this).attr('href')));
				});

				tribune.show_popup(post);
			}

			if (!tribune.postElements[post_id]) {
				tribune.show_popup($("<div><span class='tribune-message'>" + Drupal.t('fetching...') + "</span></div>"));
				tribune.element.addClass("tribune-reloading");
				tribune.ajax_preview_request = $.ajax({
					url: tribune.settings.json_post_url + "/" + post_id,
					dataType: "json",
					success: function(json) {
						if (json.posts) {
							for (post_id in json.posts) {highlight_post($(json.posts[post_id].text));}
						} else {
							tribune.hide_popup();
						}
					},
					complete: function() {tribune.element.removeClass("tribune-reloading");},
				});
			} else {
				highlight_post(tribune.postElements[post_id]);
			}
		}

		tribune.set_clock_on_hovers = function(post_id, clock) {
			clock.hover(
				function() {
					$(this).children("link[rel=clock-target]").each(function() {
						var target_id = $(this).attr("href").split('-')[1];
						tribune.highlight_main_post(target_id);
					});
				},
				function() {
					tribune.unhighlight_all_posts();
				}
			);
		};

		tribune.set_post_on_hovers = function(post) {
			post.find('a.tribune-clock').each(function() {
				tribune.set_clock_on_hovers(post.attr("value"), $(this));
			});
		}

		tribune.set_on_hovers = function(nodes) {
			nodes.filter("li.tribune-post").each(function() {
				tribune.set_post_on_hovers($(this));
			});

			nodes.find(".tribune-first-clock").click(function() {
				tribune.insert_into_palmipede($(this).text() + " ");

				return false;
			});
			
			nodes.find(".tribune-first-clock").hover(
				function() {tribune.highlight_main_post($(this).parent().attr("value"));},
				function() {tribune.unhighlight_all_posts();}
			);

			nodes.find(".tribune-login").click(function() {
				tribune.insert_into_palmipede($(this).text() + "< ");
			});

			nodes.find(".tribune-moderate-post a, .tribune-unmoderate-post a").click(function() {
				$.ajax({
					url: $(this).attr('href'),
					success: function() {tribune.load_new_posts()}
				});

				return false;
			});
		}

		tribune.start_timer = function() {
			tribune.timer = setInterval("Drupal.tribune.tribunes['" + tribune.id + "'].load_new_posts()", tribune.settings.reload_delay * 1000);
		}

		tribune.stop_timer = function() {
			clearInterval(tribune.timer);
		}

		tribune.update_submit_label = function() {
			if (tribune.palmipede.val() != '') {
				$("#form-tribune-" + tribune.id + " form input[type=submit]").val(Drupal.t('Post'));
			} else {
				$("#form-tribune-" + tribune.id + " form input[type=submit]").val(Drupal.t('Reload'));
			}
		}

		tribune.form.submit(function() {
			if (tribune.palmipede.val()) {
				$.ajax({
					url: tribune.settings.post_url,
					type: "POST",
					data: {message: tribune.palmipede.val()},
					success: function() {tribune.clear_error(); tribune.load_new_posts(); tribune.palmipede.val(""); tribune.palmipede.change();},
					error: tribune.set_error
				});
			} else {
				tribune.load_new_posts();
			}

			return false;
		});

		tribune.error.click(function() {tribune.clear_error();});

		tribune.palmipede.keyup(tribune.update_submit_label).keyup();
		tribune.palmipede.change(tribune.update_submit_label);

    tribune.palmipede.keyup(tribune.show_preview);

		tribune.set_on_hovers(tribune.pinnipede.children("li"));
		tribune.settings.after_reload = [tribune.set_on_hovers];

		tribune.start_timer();

		tribune.palmipede.get(0).focus();

		Drupal.tribune.tribunes[tribune.id] = tribune;
	})(element_id);

	element_id = null;
};

