<?php
// vim:filetype=php expandtab tabstop=2 softtabstop=2 shiftwidth=2 autoindent smartindent

function theme_tribunes_list(array $tribunes) {
  $header = array(
    'id',
    'name',
    'private',
    'administer',
    'archive',
    'delete',
  );

  $lines = array();

  foreach ($tribunes as $tribune) {
    $class = '';

    if ($tribune->private) {
      $class .= ' tribune-private';
    } else {
      $class .= ' tribune-public';
    }

    if ($tribune->archived) {
      $class .= ' tribune-archived';

      $archive_link = l('unarchive', 'tribune/'.$tribune->tid.'/unarchive',
        array(
          'query' => drupal_get_destination(),
          'attributes' => array(
            'title' => t('Re-enable posting to this tribune.'),
          ),
        )
      );
    } else {
      $archive_link = l('archive', 'tribune/'.$tribune->tid.'/archive',
        array(
          'query' => drupal_get_destination(),
          'attributes' => array(
            'title' => t('Once archived, this tribune will be read-only; this is reversible.'),
          ),
        )
      );
    }

    $name_link = tribune_user_can_read($tribune) ? l($tribune->name, 'tribune/'.$tribune->tid) : $tribune->name;
    $edit_link = tribune_user_can_administer($tribune) ? l('administer', 'tribune/'.$tribune->tid.'/admin', array('query' => drupal_get_destination())) : "";
    $delete_link = tribune_user_can_delete($tribune) ? l('delete', 'tribune/'.$tribune->tid.'/delete', array('query' => drupal_get_destination())) : "";

    $lines[] = array(
      'class' => $class,
      'data' => array(
        $tribune->tid,
        $name_link,
        $tribune->private ? 'yes' : 'no',
        $edit_link,
        $archive_link,
        $delete_link,
      ),
    );
  }

  return theme('table', $header, $lines);
}

function template_preprocess_tribune_feed(&$vars) {
  $tribune = $vars['tribune'];
  $posts = $vars['posts'];

  if ($posts === NULL) {
    $posts = $tribune->history_size;
  }

  _tribune_module_invoke_all_with_references('tribune_render_feed', $tribune);

  if (!isset($tribune->posts)) {
    $tribune->posts = tribune_load_posts($tribune, $posts);
  }

  usort($tribune->posts, '_tribune_sort_posts_by_post_id');

  $vars['posts'] = '';
  if (count($tribune->posts) > 0) foreach ($tribune->posts as $post) {
    $vars['posts'] .= theme('tribune feed post', $post, $tribune);
  }

  return $vars;
}

function template_preprocess_tribune_feed_post(&$vars) {
  $post = $vars['post'];
  $tribune = $vars['tribune'];
  if (tribune_post_belongs_to($post, $user)) {
    $vars['self'] = 'true';
  } else {
    $vars['self'] = 'false';
  }
  if (tribune_is_post_an_answer($post)) {
    $vars['answer'] = 'true';
  } else {
    $vars['answer'] = 'false';
  }

  $vars['attributes']['referenced-by'] = '';
  $references = array();
  foreach (tribune_get_post_references($post) as $post_id => $clocks) {
    foreach ($clocks as $clock_id) {
      $references[] = $post_id . '#' . $clock_id;
    }
  }
  $vars['referenced_by'] = implode('|', $references);

  $vars['post_id'] = $post->post_id;
  $vars['message'] = $post->message;
  $vars['clock'] = tribune_format_clock($post->time, $post->rank);
  $vars['time'] = $post->time;
  $vars['login'] = $post->login;
  $vars['info'] = $post->info;

  return $vars;
}

function template_preprocess_tribune(&$vars) {
  $tribune = $vars['tribune'];
  $posts = $vars['posts'];
  $show_help = $vars['show_help'];
  $show_form = $vars['show_form'];
  $show_pager = $vars['show_pager'];

  if ($posts === NULL) {
    $posts = $tribune->history_size;
  }

  // We can not use the tribune id as the HTML element id: what if
  // the same tribune is displayed twice on a single page? Once in
  // a node and once in a block, for example. So we have to create
  // a really unique id each time a tribune is displayed in a page
  // and remember these ids across calls (hence the static array).
  static $element_ids = array();

  do {$element_id = 'tribune-'.rand();}
  while (isset($element_ids[$element_id]));

  $element_ids[$element_id] = $element_id;
  $vars['element_id'] = $element_id;

  drupal_add_css(drupal_get_path('module', 'tribune') .'/css/tribune.css');
  drupal_add_js(
    array(
      'tribune' => array(
        'tribunes' => array(
          $element_id => array(
            'history_size' => $posts,
            'reload_delay' => $tribune->reload_delay,
            'time' => $_SERVER['REQUEST_TIME'],
            'post_url' => url('tribune/'.$tribune->tid.'/post'),
            'json_posts_url' => url('tribune/'.$tribune->tid.'/json/new-posts'),
            'json_post_url' => url('tribune/'.$tribune->tid.'/json/post'),
          ),
        ),
      ),
    ), 'setting'
  );
  drupal_add_js(drupal_get_path('module', 'tribune') .'/js/tribune.js');
  drupal_add_link(array(
    'href' => url('tribune/' . $tribune->tid . '/feed.xml'),
    'rel' => 'tribune feed',
    'type' => 'text/xml',
    'title' => t('XML feed address for remote clients.'),
  ));

  if (tribune_user_can_post($tribune, $user)) {
    drupal_add_link(array(
      'href' => url('tribune/' . $tribune->tid . '/post'),
      'rel' => 'tribune post',
      'title' => t('Submit address for remote clients.'),
    ));
  }

  _tribune_module_invoke_all_with_references('tribune_render', $tribune);

  if ($tribune->archived) {
    $show_form = FALSE;
  }

  if (!isset($tribune->posts)) {
    if ($show_pager and $tribune->pager) {
      $page = isset($_GET['page']) ? $_GET['page'] : 0;

      if ($page > 0) {
        $show_form = FALSE;
        $show_help = FALSE;
      }

      $tribune->posts = tribune_load_posts($tribune, $posts, TRUE);
    } else {
      $tribune->posts = tribune_load_posts($tribune, $posts);
    }
  }

  $vars['posts'] = '';
  if (count($tribune->posts) > 0) foreach ($tribune->posts as $post) {
    $vars['posts'] .= theme('tribune post', $post, $tribune);
  }

  if ($show_form) {
    $vars['form'] = drupal_get_form('tribune_post_form', $tribune);
  } else {
    $vars['form'] = '';
  }

  if ($show_help) {
    $vars['description']  = '<div class="description">';
    $vars['description'] .= t('The maximum length for a message is @max_length characters', array('@max_length' => $tribune->max_message_size));
    $vars['description'] .= '</div>';
    $vars['description'] .= '<div class="description">';
    $vars['description'] .= t('You can format your message using some HTML tags. The allowed tags are &lt;i&gt;, &lt;b&gt;, &lt;u&gt;, &lt;s&gt; and &lt;tt&gt;');
    $vars['description'] .= '</div>';
    $vars['description'] .= '<div class="description">';
    $vars['description'] .= t('URLs will be converted to links, but make sure you start them with "http://"');
    $vars['description'] .= '</div>';
  } else {
    $vars['description'] = '';
  }

  if ($show_pager) {
    $vars['pager'] = theme('pager', array(), $posts);
  } else {
    $vars['pager'] = '';
  }

  return $vars;
}

function template_preprocess_tribune_post(&$vars) {
  $post = $vars['post'];
  $tribune = $vars['tribune'];
  $vars['post_classes'] = 'tribune-post';
  if (tribune_post_belongs_to($post, $user)) {
    $vars['post_classes'] .= ' tribune-self-post';
  }
  if (tribune_is_post_an_answer($post)) {
    $vars['post_classes'] .= ' tribune-answer';
  }
  if ($post->moderated) {
    $vars['post_classes'] .= ' tribune-moderated';
  }

  $vars['moderation_link'] = '';
  if (tribune_user_can_moderate($tribune, $user)) {
    if ($post->moderated) {
      $vars['moderation_link'] .= "<span class='tribune-unmoderate-post' title='". t('Undelete this post') ."'>";
      $vars['moderation_link'] .= l('<span>'.t("Undelete").'</span>',
        'tribune/'. $tribune->tid .'/moderation/undelete/'. $post->pid,
        array('html' => TRUE, 'attributes' => array('target' => '_blank'))
      );
      $vars['moderation_link'] .= "</span>\n";
    }
    else {
      $vars['moderation_link'] .= "<span class='tribune-moderate-post' title='". t('Delete this post') ."'>";
      $vars['moderation_link'] .= l('<span>'.t("Delete").'</span>',
        'tribune/'. $tribune->tid .'/moderation/delete/'. $post->pid,
        array('html' => TRUE, 'attributes' => array('target' => '_blank'))
      );
      $vars['moderation_link'] .= "</span>\n";
    }
  }

  $vars['references'] = '';
  foreach (tribune_get_post_references($post) as $post_id => $clocks) {
    foreach ($clocks as $clock_id) {
      $vars['references'] .= "<link rel='post-reference' href='#post-{$post_id}-clock-{$clock_id}' />";
    }
  }

  if (strlen($post->login) > 0) {
    $vars['nickname'] = check_plain($post->login);
    $vars['nickname_classes'] = 'tribune-login';
  }
  else {
    $vars['nickname'] = t('Anonymous');
    $vars['nickname_classes'] = 'tribune-info';
  }

  $vars['post_id'] = $post->post_id;
  $vars['message'] = $post->html_message;
  $vars['clock'] = tribune_format_clock($post->time, $post->rank);
  $vars['login'] = $post->login;
  $vars['info'] = $post->info;

  return $vars;
}

function _tribune_sort_posts_by_post_id($post_a, $post_b) {
  return $post_a->post_id > $post_b->post_id ? -1 : 1;
}

